@@ -1,16 +1,16 @@
 package App::perlbrew;
 use strict;
 use warnings;
-use 5.8.0;
+use 5.008;
 use Getopt::Long ();
 use File::Spec::Functions qw( catfile );
 
-our $VERSION = "0.10";
+our $VERSION = "0.11";
 our $CONF;
 
 my $ROOT         = $ENV{PERLBREW_ROOT} || "$ENV{HOME}/perl5/perlbrew";
 my $CONF_FILE    = catfile( $ROOT, 'Conf.pm' );
-my $CURRENT_PERL = "$ROOT/perls/current";
+my $CURRENT_PERL = $ENV{PERLBREW_PERL};
 
 my @GETOPT_CONFIG = (
     'pass_through',
@@ -24,7 +24,7 @@ my @GETOPT_SPEC = (
     'verbose|v',
     'as=s',
 
-    'help|?',
+    'help|h',
     'version',
 
     # options passed directly to Configure
@@ -33,6 +33,10 @@ my @GETOPT_SPEC = (
     'A=s@',
 );
 
+sub uniq(@) {
+    my %a;
+    grep { ++$a{$_} == 1 } @_;
+}
 
 sub new {
     my($class, @argv) = @_;
@@ -46,9 +50,12 @@ sub new {
         A => [],
     );
 
+    # build a local @ARGV to allow us to use an older
+    # Getopt::Long API in case we are building on an older system
+    local (@ARGV) = @argv;
     Getopt::Long::Configure(@GETOPT_CONFIG);
-    Getopt::Long::GetOptionsFromArray(\@argv, \%opt, @GETOPT_SPEC)
-        or run_command_help(1);
+    Getopt::Long::GetOptions( \%opt, @GETOPT_SPEC )
+      or run_command_help(1);
 
     # fix up the effect of 'bundling'
     foreach my $flags (@opt{qw(D U A)}) {
@@ -57,7 +64,8 @@ sub new {
         }
     }
 
-    $opt{args} = \@argv;
+    $opt{args} = \@ARGV;
+
     return bless \%opt, $class;
 }
 
@@ -81,10 +89,18 @@ sub run_command {
     if($self->{version}) {
         $x = 'version';
     }
-    elsif(!$x || $self->{help}) {
+    elsif(!$x) {
         $x = 'help';
+        @args = (0, ($self->{help} ? 2 : 0));
     }
-    my $s = $self->can("run_command_$x") or die "Unknown command: `$x`. Typo?\n";
+
+    my $s = $self->can("run_command_$x");
+    unless ($s) {
+        $x =~ s/-/_/;
+        $s = $self->can("run_command_$x");
+    }
+
+    die "Unknown command: `$x`. Typo?\n" unless $s;
     $self->$s(@args);
 }
 
@@ -98,9 +114,9 @@ VERSION
 }
 
 sub run_command_help {
-    my ( $self, $status ) = @_;
+    my ($self, $status, $verbose) = @_;
     require Pod::Usage;
-    Pod::Usage::pod2usage(defined $status ? $status : 1);
+    Pod::Usage::pod2usage(-verbose => $verbose||0, -exitval => (defined $status ? $status : 1));
 }
 
 sub run_command_init {
@@ -110,8 +126,67 @@ sub run_command_init {
         "$ROOT/bin"
     );
 
+    open BASHRC, "> $ROOT/etc/bashrc";
+    print BASHRC <<'RC';
+if [[ -f $HOME/.perlbrew/init ]]; then
+    source $HOME/.perlbrew/init
+fi
+
+export PATH_WITHOUT_PERLBREW=$(perl -e 'print join ":", grep { index($_, $ENV{PERLBREW_ROOT}) } split/:/,$ENV{PATH};')
+export PATH=$PERLBREW_PATH:$PATH_WITHOUT_PERLBREW
+
+perlbrew () {
+    case $1 in
+        (use)
+            if [[ -x "$PERLBREW_ROOT/perls/$2/bin/perl" ]]; then
+                eval $(command perlbrew env $2)
+                export PATH=$PERLBREW_PATH:$PATH_WITHOUT_PERLBREW
+            else
+                echo "$2 is not installed";
+            fi
+            ;;
+
+        (switch)
+            if [[ ! -d $HOME/.perlbrew ]]; then
+                mkdir -p $HOME/.perlbrew
+            fi
+
+            if [[ -x "$PERLBREW_ROOT/perls/$2/bin/perl" ]]; then
+                echo '# DO NOT EDIT THIS FILE' > $HOME/.perlbrew/init
+                command perlbrew env $2 >> $HOME/.perlbrew/init
+                source $HOME/.perlbrew/init
+                export PATH=$PERLBREW_PATH:$PATH_WITHOUT_PERLBREW
+            else
+                echo "$2 is not installed";
+            fi
+            ;;
+
+        (off)
+            if [[ ! -d $HOME/.perlbrew ]]; then
+                mkdir -p $HOME/.perlbrew
+            fi
+
+            unset PERLBREW_PERL
+            command perlbrew off
+
+            echo '# DO NOT EDIT THIS FILE' > $HOME/.perlbrew/init
+            command perlbrew env >> $HOME/.perlbrew/init
+            source $HOME/.perlbrew/init
+            export PATH=$PERLBREW_PATH:$PATH_WITHOUT_PERLBREW
+            ;;
+
+        (*)
+            command perlbrew $*
+            ;;
+    esac
+    hash -r
+}
+
+
+RC
+    close BASHRC;
+
     system <<RC;
-echo 'export PATH=$ROOT/bin:$ROOT/perls/current/bin:\${PATH}' > $ROOT/etc/bashrc
 echo 'setenv PATH $ROOT/bin:$ROOT/perls/current/bin:\$PATH' > $ROOT/etc/cshrc
 RC
 
@@ -251,6 +326,7 @@ HELP
 
         my @d_options = @{ $self->{D} };
         my @u_options = @{ $self->{U} };
+        my @a_options = @{ $self->{A} };
         my $as = $self->{as} || ($dist_git_describe ? "perl-$dist_git_describe" : $dist);
         unshift @d_options, qq(prefix=$ROOT/perls/$as);
         push @d_options, "usedevel" if $dist_version =~ /5\.1[13579]|git/ ? "-Dusedevel" : "";
@@ -286,8 +362,9 @@ INSTALL
             "rm -f config.sh Policy.sh",
             "sh Configure $configure_flags " .
                 join( ' ',
-                    ( map { "-D$_" } @d_options ),
-                    ( map { "-U$_" } @u_options ),
+                    ( map { qq{'-D$_'} } @d_options ),
+                    ( map { qq{'-U$_'} } @u_options ),
+                    ( map { qq{'-A$_'} } @a_options ),
                 ),
             $dist_version =~ /^5\.(\d+)\.(\d+)/
                 && ($1 < 8 || $1 == 8 && $2 < 9)
@@ -303,22 +380,33 @@ INSTALL
 
         delete $ENV{$_} for qw(PERL5LIB PERL5OPT);
 
-        print !system($cmd) ? <<SUCCESS : <<FAIL;
+        if (!system($cmd)) {
+            if ($dist_version =~ /5\.1[13579]|git/) {
+                $self->run_command_symlink_executables($as);
+            }
+
+            $self->run_command_install_cpanm($as);
+
+            print <<SUCCESS;
 Installed $dist as $as successfully. Run the following command to switch to it.
 
   perlbrew switch $as
 
 SUCCESS
+        }
+        else {
+            print <<FAIL;
 Installing $dist failed. See $self->{log_file} to see why.
 If you want to force install the distribution, try:
 
   perlbrew --force install $dist_name
 
 FAIL
+        }
     }
 }
 
-sub calc_installed {
+sub get_installed_perls {
     my $self    = shift;
     my $current = readlink("$ROOT/perls/current");
 
@@ -327,31 +415,39 @@ sub calc_installed {
     for (<$ROOT/perls/*>) {
         next if m/current/;
         my ($name) = $_ =~ m/\/([^\/]+$)/;
-        push @result, { name => $name, is_current => ($name eq $current ? 1 : 0)  };
+        push @result, { name => $name, is_current => ($CURRENT_PERL && ($name eq $CURRENT_PERL)) };
     }
 
-    my $current_perl_executable = readlink("$ROOT/bin/perl");
-    for ( grep { -f $_ && -x $_ } map { "$_/perl" } split(":", $ENV{PATH}) ) {
+    my $current_perl_executable = readlink("$ROOT/bin/perl") || `which perl`;
+    $current_perl_executable =~ s/\n$//;
+
+    for ( uniq grep { -f $_ && -x $_ } map { "$_/perl" } split(":", $ENV{PATH}) ) {
         push @result, {
             name       => $_,
-            is_current => ($_ eq $current_perl_executable ? 1 : 0),
-        };
+            is_current => $current_perl_executable && ($_ eq $current_perl_executable)
+        } unless index($_, $ROOT) == 0;
     }
 
     return @result;
 }
 
-sub run_command_installed {
+sub run_command_list {
     my $self = shift;
-    my @installed = $self->calc_installed(@_);
+    my @installed = $self->get_installed_perls(@_);
 
     for my $installed (@installed) {
         my $name = $installed->{name};
         my $cur  = $installed->{is_current};
-        print $name, ($cur ? '(*)' : ''), "\n";
+        print $cur ? '* ': '  ', $name, "\n";
     }
 }
 
+sub run_command_installed {
+    print STDERR "\nDEPRECATION WARNING: `perlbrew installed` will be deprecated soon. Please run `perlbrew list` instead.\n\n";
+    sleep 1;
+    &run_command_list;
+}
+
 sub run_command_switch {
     my ( $self, $dist ) = @_;
 
@@ -418,15 +514,20 @@ sub run_command_mirror {
             if ( $test == int $test ) {
                 my $remaining = $#mirrors - $id;
                 my $ask = "Select a mirror by number or press enter to see the rest "
-                        . "($remaining more) [q to quit]";
+                        . "($remaining more) [q to quit, m for manual entry]";
                 my $val = ExtUtils::MakeMaker::prompt( $ask );
                 next MIRROR if ! $val;
                 last MIRROR if $val eq 'q';
                 $select = $val;
-                if ( ! $select || $select - 1 > $#mirrors ) {
+		if($select eq 'm') {
+                    my $url  = ExtUtils::MakeMaker::prompt("Enter the URL of your CPAN mirror:");
+		    my $name = ExtUtils::MakeMaker::prompt("Enter a Name: [default: My CPAN Mirror]") || "My CPAN Mirror";
+		    $select = { name => $name, url => $url };
+		}
+                elsif ( ! $select || $select - 1 > $#mirrors ) {
                     die "Bogus mirror ID: $select";
                 }
-                $select = $mirrors[$select - 1];
+                $select = $mirrors[$select - 1] unless ($select eq 'm');
                 die "Mirror ID is invalid" if ! $select;
                 last MIRROR;
             }
@@ -440,6 +541,54 @@ sub run_command_mirror {
     return;
 }
 
+sub run_command_env {
+    my($self, $perl) = @_;
+
+    my %env = (
+        VERSION => $VERSION,
+        PATH => "$ROOT/bin",
+        ROOT => $ROOT
+    );
+
+    if ($perl && -d "$ROOT/perls/$perl/bin") {
+        $env{PERL} = $perl;
+        $env{PATH} .= ":$ROOT/perls/$perl/bin";
+    }
+
+    if ($ENV{SHELL} =~ /(ba|z)sh$/) {
+        while (my ($k, $v) = each(%env)) {
+            print "export PERLBREW_$k=$v\n";
+        }
+    }
+    else {
+        while (my ($k, $v) = each(%env)) {
+            print "setenv PERLBREW_$k $v\n";
+        }
+    }
+}
+
+sub run_command_symlink_executables {
+    my($self, $perl) = @_;
+
+    return "" unless $perl;
+
+    for my $executable (<$ROOT/perls/$perl/bin/*>) {
+        my ($name, $version) = $executable =~ m/bin\/(.+?)(5\.\d.*)?$/;
+        system("ln -fs $executable $ROOT/perls/$perl/bin/$name") if $version;
+    }
+}
+
+sub run_command_install_cpanm {
+    my ($self, $perl) = @_;
+    my $body = $self->_http_get('http://github.com/miyagawa/cpanminus/raw/master/cpanm');
+
+    open my $CPANM, "> $ROOT/bin/cpanm";
+    print $CPANM $body;
+    close $CPANM;
+    chmod 0755, "$ROOT/bin/cpanm";
+    print "cpanm is installed to $ROOT/bin/cpanm\n" if $self->{verbose};
+}
+
 sub _http_get {
     my ($self, $url, $cb, $header) = @_;
     require HTTP::Lite;
@@ -523,15 +672,15 @@ App::perlbrew - Manage perl installations in your $HOME
     perlbrew mirror
 
     # Install some Perls
-    perlbrew install perl-5.12.1
+    perlbrew install perl-5.12.2
     perlbrew install perl-5.8.1
-    perlbrew install perl-5.11.5
+    perlbrew install perl-5.13.5
 
     # See what were installed
     perlbrew installed
 
     # Switch perl in the $PATH
-    perlbrew switch perl-5.12.1
+    perlbrew switch perl-5.12.2
     perl -v
 
     # Switch to another version
@@ -545,7 +694,7 @@ App::perlbrew - Manage perl installations in your $HOME
     perlbrew off
 
     # Use 'switch' command to turn it back on.
-    perlbrew switch perl-5.12.1
+    perlbrew switch perl-5.12.2
 
 =head1 DESCRIPTION
 
@@ -580,7 +729,7 @@ a C<PERLBREW_ROOT> environment variable before you run C<./perlbrew install>.
     ./perlbrew install
 
 The downloaded perlbrew is a self-contained standalone program that
-embed all non-core modules it uses. It should be runnable with perl
+embeds all non-core modules it uses. It should be runnable with perl
 5.8 or later versions of perl.
 
 You may also install perlbrew from CPAN with cpan / cpanp / cpanm:
@@ -593,7 +742,7 @@ executed with your current perl.
 NOTICE. When you install or upgrade perlbrew with cpan / cpanp /
 cpanm, make sure you are not using one of the perls brewed with
 perlbrew. If so, the `perlbrew` executable you just installed will not
-be available after you swith to other perls. You might not be able to
+be available after you switch to other perls. You might not be able to
 invoke further C<perlbrew> commands after so because the executable
 C<perlbrew> is not in your C<PATH> anymore. Installing it again with
 cpan can temporarily solve this problem. To ensure you are not using
@@ -602,7 +751,7 @@ a perlbrewed perl, run C<perlbrew off> before upgrading.
 
 It should be relatively safe to install C<App::perlbrew> with system
 cpan (like C</usr/bin/cpan>) because then it will be installed under a
-system PATH like C</usr/bin>, which is not effected by C<perlbrew switch>
+system PATH like C</usr/bin>, which is not affected by C<perlbrew switch>
 command.
 
 Again, it is recommended to let C<perlbrew> install itself. It's
@@ -614,7 +763,7 @@ Please read the program usage by running
 
     perlbrew
 
-(No arguments.) To read a more detail one:
+(No arguments.) To read a more detailed one:
 
     perlbrew -h
 
@@ -631,7 +780,7 @@ It will make sure that your current perl in the PATH is pointing
 to C</usr/bin/perl>.
 
 As a matter of fact the C<switch> command checks whether the given
-argument is an executale or not, and create a symlink named 'perl' to
+argument is an executable or not, and create a symlink named 'perl' to
 it if it is. If you really want to you are able to do:
 
     perlbrew switch /usr/bin/perl6
@@ -639,6 +788,14 @@ it if it is. If you really want to you are able to do:
 But maybe not. After running this you might not be able to run
 perlbrew anymore. So be careful not making mistakes there.
 
+=head1 PROJECT DEVELOPMENT
+
+perlbrew project uses PivotalTracker for task tracking:
+L<http://www.pivotaltracker.com/projects/129997>. You may also report
+bugs to github L<http://github.com/gugod/App-perlbrew/issues> or RT
+<https://rt.cpan.org/Dist/Display.html?Queue=App-perlbrew>. They will
+be definitely reviewed and handled.
+
 =head1 AUTHOR
 
 Kang-min Liu  C<< <gugod@gugod.org> >>
